/**
 * @file Clock.h
 * @author Richard Buckley <richard.buckley@ieee.org>
 * @version 1.0
 * @date 2021-12-01
 */

#pragma once

#include "Text.h"
#include "TimerTick.h"
#include "cpp_local_time/LocalTime.h"

namespace rose {

    class Clock : public Widget {
    protected:
        static constexpr std::string_view ClockClass{"Clock"};
        std::string HoursMinutesFmt = "%R";
        std::string SecondsFmt = "%S";
        std::string ZoneFmt = "%Z";
        FontSize mFontSize{FontSize::LARGE};
        FontSelection mFontSelection{FontSelection::FIXED_BOLD};

        LineTypeSetter lineTypeSetter{};
        std::shared_ptr<ColumnTypeSetter> columnTypeSetter{};

        TickProtocol::slot_type second_slot{};
        std::string mTimeZone{"EST"};    ///< The time zone if not empty.
        bool mLocalTime{false};
        bool mDisplaySeconds{true};
        std::unique_ptr<cpp_local_time::LocalTime> mLocalTimeConvert{};

        void updateTimeDisplay(int seconds);

    private:
        void getResources();

        void initialize();

    public:
        Clock() : Widget(property::Class, ClockClass) {
            initialize();
        }

        Clock(const property_key &key, const property_value &value) : Widget(key, value) {
            initialize();
        }

        ~Clock() override = default;

        void
        draw(gm::Context &context, const Position<int> &trunkPosition, const Size<int> &trunkSize) override;

        void
        layout(gm::Context &context, const Position<int> &trunkPosition, const Size<int> &trunkSize) override;

    };

    class Date : public Widget {
    protected:
        static constexpr std::string_view DateClass{"Date"};
        std::string FullDateFmt ="%a %b %d, %y";
        FontSize mFontSize{FontSize::LARGE};
        FontSelection mFontSelection{FontSelection::FIXED_BOLD};

        LineTypeSetter lineTypeSetter{};

        TickProtocol::slot_type second_slot{};
        std::string mTimeZone{"EST"};    ///< The time zone if not empty.
        bool mLocalTime{false};
        std::unique_ptr<cpp_local_time::LocalTime> mLocalTimeConvert{};

        void updateDateDisplay();

    private:
        void getResources();

        void initialize();

    public:
        Date() : Widget(property::Class, DateClass) {
            initialize();
        }

        Date(const property_key &key, const property_value &value) : Widget(key, value) {
            initialize();
        }

        ~Date() override = default;

        void
        draw(gm::Context &context, const Position<int> &trunkPosition, const Size<int> &trunkSize) override;

        void
        layout(gm::Context &context, const Position<int> &trunkPosition, const Size<int> &trunkSize) override;
    };
}

