/**
 * @file DrawingPrimitives.h
 * @author Richard Buckley <richard.buckley@ieee.org>
 * @version 1.0
 * @date 2021-12-28
 */

#pragma once

#include <SDL2/SDL2_gfxPrimitives.h>
#include "Color.h"
#include "Context.h"

namespace rose {
        bool aaEllipse(gm::Context& context, const Position<int>& center, const Size<int>& radii, color::RGBA rgba) {
            gm::DrawColorGuard drawColorGuard(context, rgba);
            auto color = rgba.toSdlColor();
            if (aaellipseRGBA(context.get(), static_cast<int16_t>(center.x), static_cast<int16_t>(center.y),
                              static_cast<int16_t>(radii.w), static_cast<int16_t>(radii.h),
                              color.r, color.g, color.b, color.a)) {
                return RoseError(ErrorStatus::GFX_ERROR);
            }
            return true;
        }

        bool ellipse (gm::Context& context, const Position<int>& center, const Size<int>& radii, color::RGBA rgba, bool fill=false) {
            gm::DrawColorGuard drawColorGuard(context, rgba);
            auto color = rgba.toSdlColor();
            if (fill) {
                if (filledEllipseRGBA(context.get(),  static_cast<int16_t>(center.x), static_cast<int16_t>(center.y),
                                static_cast<int16_t>(radii.w), static_cast<int16_t>(radii.h),
                                color.r, color.g, color.b, color.a)) {
                    return RoseError(ErrorStatus::GFX_ERROR);
                }
            } else {
                if (ellipseRGBA(context.get(),  static_cast<int16_t>(center.x), static_cast<int16_t>(center.y),
                                static_cast<int16_t>(radii.w), static_cast<int16_t>(radii.h),
                                color.r, color.g, color.b, color.a)) {
                    return RoseError(ErrorStatus::GFX_ERROR);
                }
            }
            return true;
        }
}


