/**
 * @file Image.h
 * @author Richard Buckley <richard.buckley@ieee.org>
 * @version 1.0
 * @date 2021-12-06
 */

#pragma once

#include <filesystem>
#include "Widget.h"
#include "Sdl++/Texture.h"
#include "Sdl++/TextureCache.h"


namespace rose {
    namespace fs = ::std::filesystem;

    struct ImageClass {
        unique_id_type imageClass{0};

        constexpr auto operator<=>(const ImageClass &) const = default;

        constexpr auto operator<=>(unique_id_type id) const { return imageClass <=> id; }

        [[nodiscard]] constexpr explicit operator bool() const noexcept { return imageClass != 0; }
    };

    /**
     * @class Image
     * @brief Display image textures from the TextureCache.
     */
    class Image : public Widget {
    protected:

        TextureCache &mTextureCache;
        unique_id_type mCacheId{};
        std::vector<unique_id_type> mImageClassList{};

    public:
        Image();

        ~Image() override = default;

        void
        draw(gm::Context &context, const Position<int> &trunkPosition, const Size<int> &trunkSize) override;

        void
        layout(gm::Context &context, const Position<int> &trunkPosition, const Size<int> &trunkSize) override;

        template<class Parameter>
        void setParameter(Parameter parameter) {
            Widget::setParameter(parameter);
        }

        bool handleEvent(const Event& event) override;
    };

    template<>
    inline void Image::setParameter<unique_id_type>(unique_id_type parameter) {
        mCacheId = parameter;
    }

    template<>
    inline void Image::setParameter<std::vector<unique_id_type>>(std::vector<unique_id_type> parameter) {
        if (mImageClassList != parameter) {
            mImageClassList = parameter;
            redrawBackground();
        }
    }
}

