/**
 * @file Layout.h
 * @author Richard Buckley <richard.buckley@ieee.org>
 * @version 1.0
 * @date 2021-12-02
 */

#pragma once

#include "Widget.h"

namespace rose {

    class Container;

    /**
     * @class Layout
     * @brief
     */
    class Layout : public std::enable_shared_from_this<Layout> {
    protected:

    public:
        Layout() = default;

        virtual ~Layout() = default;

        template<typename...Args>
        explicit Layout(Args...args) : Layout() {
            ((setParameter(args)), ...);
        }

        template<class Parameter>
        void setParameter(Parameter) {
            throw std::runtime_error("Parameter not valid for Layout type.");
        }

        virtual Size<int> layout(Container &container, gm::Context &context,
                                 const Position<int> &trunkPosition, const Size<int> &trunkSize);
    };

    class LinearLayout : public Layout {
    protected:
        Orientation mOrientation{Orientation::Vertical};

    public:
        LinearLayout() = default;

        ~LinearLayout() override = default;

        template<typename...Args>
        explicit LinearLayout(Args...args) : Layout() {
            ((setParameter(args)), ...);
        }

        template<class Parameter>
        void setParameter(Parameter parameter) {
            Layout::setParameter(parameter);
        }

        Size<int> layout(Container &container, gm::Context &context,
                         const Position<int> &trunkPosition, const Size<int> &trunkSize) override;
    };

    template<>
    inline void LinearLayout::setParameter<Orientation>(Orientation parameter) {
        mOrientation = parameter;
    }
}

