/**
 * @file ValueQuad.d
 * @author Richard Buckley <richard.buckley@ieee.org>
 * @version 1.0
 * @date 2021-11-27
 */

#pragma once

#include <tuple>
#include "Size.h"
#include "Position.h"

namespace rose {

    /**
     * @struct Margin
     * @brief Abstraction of space consumed around an object for space, borders, etc.
     */
    template<typename T=int>
    struct Margin {
        T top{}, bot{}, left{}, right{};

        constexpr Margin() noexcept = default;

        constexpr explicit Margin(T p) noexcept : top(p), bot(p), left(p), right(p) {}

        constexpr Margin(const Margin<T>& margin) noexcept = default;

        constexpr Margin(Margin&& margin) noexcept = default;

        constexpr Margin<T>& operator = (const Margin<T>& margin) noexcept = default;

        constexpr Margin<T>& operator = (Margin && margin) noexcept = default;

        constexpr Margin(T h, T v) noexcept : Margin(v, v, h, h) {}

        constexpr Margin(T top, T bot, T left, T right) noexcept : top(top), bot(bot), left(left), right(right) {}

        constexpr Margin(const Position<T>& pos, const Size<T>& size) noexcept : Margin(pos.x, pos.y, size.w, size.h) {}

        constexpr explicit Margin(const std::array<T,4>& array) noexcept {
            top = array[0]; bot = array[1]; left = array[2]; right = array[3];
        }

        constexpr Margin<T>& operator = (const std::array<T,4>& array) noexcept {
            top = array[0]; bot = array[1]; left = array[2]; right = array[3];
            return *this;
        }

        [[nodiscard]] constexpr T vertical() const noexcept { return top + bot; }

        [[nodiscard]] constexpr T horizontal() const noexcept { return left + right; }

        [[nodiscard]] constexpr Position<T> position() const noexcept { return Position<T>{left, top}; }

        [[nodiscard]] constexpr Size<T> size() const noexcept { return Size{horizontal(), vertical()}; }
    };
}
