/**
 * @file Metrics.h
 * @author Richard Buckley <richard.buckley@ieee.org>
 * @version 1.0
 * @date 2021-11-23
 */

#pragma once

#include <SDL.h>
#include "Math.h"
#include "Utilities.h"
#include "Position.h"
#include "Size.h"
#include "Rectangle.h"
#include "Padding.h"
#include "Margin.h"

template<typename T>
rose::Position<T> operator-(const rose::Position<T> &p, const rose::Size<T> &s) noexcept {
    rose::Position<T> r{p.x - s.w, p.y - s.h};
    return r;
}

template<typename T>
inline rose::Position<T> operator+(const rose::Position<T>& position, const rose::Size<T>& size) {
    return rose::Position<T>{position.x + size.w, position.y + size.h};
}

template<class T>
inline std::ostream& operator<<(std::ostream& strm, const rose::Size<T> &p) {
    return p.printOn(strm);
}

template<class T>
inline std::ostream& operator<<(std::ostream& strm, const rose::Position<T> &p) {
    return p.printOn(strm);
}

template<class T>
inline std::ostream& operator << (std::ostream& strm, const rose::Rectangle<T>& value) {
        return strm << '[' << value.x << ',' << value.y << ',' << value.w << ',' << value.h << ']';
}

template<class T>
inline std::ostream& operator << (std::ostream& strm, const rose::Padding<T>& value) {
    return strm << '[' << value.top << ',' << value.bot << ',' << value.left << ',' << value.right << ']';
}

template<class T>
inline std::ostream& operator << (std::ostream& strm, const rose::Margin<T>& value) {
    return strm << '[' << value.top << ',' << value.bot << ',' << value.left << ',' << value.right << ']';
}

template<class T>
inline std::ostream& operator << (std::ostream& strm, const rose::Line<T>& value) {
    return strm << value.p0 << ',' << value.p1;
}
