/**
 * @file Padding.h
 * @author Richard Buckley <richard.buckley@ieee.org>
 * @version 1.0
 * @date 2021-11-27
 */

#pragma once

#include <tuple>
#include "Size.h"
#include "Position.h"

namespace rose {

    /**
     * @struct Padding
     * @brief Abstraction of space consumed around an object for space, borders, etc.
     */
    template<typename T=int>
    struct Padding {
        T top{}, bot{}, left{}, right{};

        constexpr Padding() noexcept = default;

        constexpr explicit Padding(int p) noexcept: top(p), bot(p), left(p), right(p) {}

        constexpr Padding(const Padding<T> &Padding) noexcept = default;

        constexpr Padding(Padding &&Padding) noexcept = default;

        constexpr Padding<T> &operator=(const Padding<T> &Padding) noexcept = default;

        constexpr Padding<T> &operator=(Padding &&Padding) noexcept = default;

        constexpr Padding(T h, T v) noexcept: Padding(v, v, h, h) {}

        constexpr Padding(T top, T bot, T left, T right) noexcept: top(top), bot(bot), left(left), right(right) {}

        constexpr Padding(const Padding <T> &pos, const Size<T> &size) : Padding(pos.x, pos.y, size.w, size.h) {}

        constexpr explicit Padding(const std::array<T, 4> &array) noexcept {
            top = array[0];
            bot = array[1];
            left = array[2];
            right = array[3];
        }

        constexpr Padding<T> &operator=(const std::array<T, 4> &array) noexcept {
            top = array[0];
            bot = array[1];
            left = array[2];
            right = array[3];
            return *this;
        }

        [[nodiscard]] constexpr T vertical() const noexcept { return top + bot; }

        [[nodiscard]] constexpr T horizontal() const noexcept { return left + right; }

        [[nodiscard]] constexpr Position<T> position() const noexcept { return Position<T>{left, top}; }

        [[nodiscard]] constexpr Size<T> size() const noexcept { return Size{horizontal(), vertical()}; }
    };
}
