/**
 * @file Property.h
 * @author Richard Buckley <richard.buckley@ieee.org>
 * @version 1.0
 * @date 2021-11-22
 */

#pragma once

#include <string>
#include <string_view>
#include <utility>
#include <variant>
#include <utility>
#include <iostream>

namespace rose {

    namespace property {
        static constexpr std::string_view Name = "Name";
        static constexpr std::string_view Class = "Class";
        [[maybe_unused]] static constexpr std::string_view Style = "Style";
    }

    using property_value = std::variant<int,double,std::string,std::string_view>;
    using property_key = std::string_view;
    using proprety_type = std::pair<property_key,property_value>;

    /**
     * @class Property
     * @brief Define properties for Widgets.
     */
    class Property : public proprety_type {

    public:
        Property() = delete;

        [[maybe_unused]] explicit Property(const std::string_view name) : std::pair<property_key ,property_value>(name,0) {}

        template<typename ValueType>
        Property(const property_key name, ValueType value) : std::pair<std::string_view,property_value>() {
            first = name;
            if constexpr(std::is_same_v<ValueType,const char*>) {
                second = std::string(value);
            } else {
                second = value;
            }
        }

        static std::ostream& printOn(std::ostream& strm, const std::pair<property_key,property_value> &property);
    };
}
