/**
 * @file Screen.h
 * @author Richard Buckley <richard.buckley@ieee.org>
 * @version 1.0
 * @date 2021-12-01
 */

#pragma once

#include "Container.h"

namespace rose {

    class Screen : public Container {
    protected:
        WidgetPtr mFocusWidget{};
        std::vector<WidgetPtr> mEventChain{};
        bool mRedrawBackground{true};

    private:
        void getResources();

    public:
        static constexpr std::string_view ScreenClass = "Screen";

        Screen() : Container(property::Class, ScreenClass) {
            getResources();
        }

        Screen(const property_key key, const property_value &value) : Container(key,value) {
            getResources();
        }

        ~Screen() override = default;

        void draw(gm::Context &context, const Position<int>& trunkPosition, const Size<int>& trunkSize) override;

        void setTheme(const Theme& theme) {
            mTheme = theme;
            setRedrawBackground();
        }

        auto focusWidget() { return mFocusWidget; }

        void setRedrawBackground() { mRedrawBackground = true; }

        void clearRedrawBackground() { mRedrawBackground = false; }

        bool getRedrawBackground() const { return mRedrawBackground; }

        bool handleEvent(const Event& event) override;

        bool handleLocatedEvent(const Event &eventLocation);

        void setPerFrameProcess() override;

        void clearPerFrameProcess() override;

        bool handleKeyShortcut(const KeyShortcut &keyShortcut) override {
            if (focusWidget() && focusWidget()->handleKeyShortcut(keyShortcut))
                return true;

            if (Container::handleKeyShortcut(keyShortcut))
                return true;

            return false;
        }
    };

    using ScreenPtr = std::shared_ptr<Screen>;

}

